@@ -3,25 +3,30 @@ Utility functions for typhos
 """
 import collections
 import contextlib
+import functools
 import importlib.util
 import inspect
 import io
 import logging
+import operator
 import os
 import pathlib
 import random
 import re
 import threading
 
-from qtpy import QtCore, QtWidgets
-from qtpy.QtCore import QSize
-from qtpy.QtGui import QColor, QMovie, QPainter
-from qtpy.QtWidgets import QWidget
-
+import ophyd
 import ophyd.sim
 from ophyd import Device
 from ophyd.signal import EpicsSignalBase, EpicsSignalRO
 from pydm.exception import raise_to_operator  # noqa
+from pydm.widgets.base import PyDMWritableWidget
+from qtpy import QtCore, QtGui, QtWidgets
+from qtpy.QtCore import QSize
+from qtpy.QtGui import QColor, QMovie, QPainter
+from qtpy.QtWidgets import QWidget
+
+from typhos import plugins
 
 try:
     import happi
@@ -31,8 +36,10 @@ except ImportError:
 logger = logging.getLogger(__name__)
 MODULE_PATH = pathlib.Path(__file__).parent.resolve()
 ui_dir = MODULE_PATH / 'ui'
+ui_core_dir = ui_dir / 'core'
 GrabKindItem = collections.namedtuple('GrabKindItem',
                                       ('attr', 'component', 'signal'))
+DEBUG_MODE = bool(os.environ.get('TYPHOS_DEBUG', False))
 
 
 if happi is None:
@@ -46,38 +53,46 @@ def _get_display_paths():
         path = pathlib.Path(path).expanduser().resolve()
         if path.exists() and path.is_dir():
             yield path
-    yield ui_dir
+    yield ui_dir / 'core'
+    yield ui_dir / 'devices'
 
 
 DISPLAY_PATHS = list(_get_display_paths())
 
 
-class SignalRO(ophyd.sim.SynSignalRO):
-    def __init__(self, value=0, *args, **kwargs):
-        self._value = value
-        super().__init__(*args, **kwargs)
-        self._metadata.update(
-            connected=True,
-            write_access=False,
-        )
+if hasattr(ophyd.signal, 'SignalRO'):
+    SignalRO = ophyd.signal.SignalRO
+else:
+    # SignalRO was re-introduced to ophyd.signal in December 2019 (1f83a055).
+    # If unavailable, fall back to our previous definition:
+    class SignalRO(ophyd.sim.SynSignalRO):
+        def __init__(self, value=0, *args, **kwargs):
+            self._value = value
+            super().__init__(*args, **kwargs)
+            self._metadata.update(
+                connected=True,
+                write_access=False,
+            )
 
-    def get(self):
-        return self._value
+        def get(self):
+            return self._value
 
 
-def channel_from_signal(signal):
+def channel_from_signal(signal, read=True):
     """
     Create a PyDM address from arbitrary signal type
     """
     # Add an item
     if isinstance(signal, EpicsSignalBase):
-        return channel_name(signal._read_pv.pvname)
+        if read:
+            return channel_name(signal._read_pv.pvname)
+        return channel_name(signal._write_pv.pvname)
     return channel_name(signal.name, protocol='sig')
 
 
 def is_signal_ro(signal):
     """
-    Return whether the signal is read-only
+    Return whether the signal is read-only, based on its class.
 
     In the future this may be easier to do through improvements to
     introspection in the ophyd library. Until that day we need to check classes
@@ -179,9 +194,9 @@ class TyphosLoading(QtWidgets.QLabel):
     LOADING_TIMEOUT_MS = 10000
     loading_gif = None
 
-    def __init__(self, *args, **kwargs):
-
-        super().__init__(*args, **kwargs)
+    def __init__(self, timeout_message, *, parent=None, **kwargs):
+        self.timeout_message = timeout_message
+        super().__init__(parent=parent, **kwargs)
         self._icon_size = QSize(32, 32)
         if TyphosLoading.loading_gif is None:
             loading_path = os.path.join(ui_dir, 'loading.gif')
@@ -194,12 +209,33 @@ class TyphosLoading(QtWidgets.QLabel):
             QtCore.QTimer.singleShot(self.LOADING_TIMEOUT_MS,
                                      self._handle_timeout)
 
+        self.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
+
+    def contextMenuEvent(self, event):
+        menu = QtWidgets.QMenu(parent=self)
+
+        def copy_to_clipboard(*, text):
+            clipboard = QtWidgets.QApplication.instance().clipboard()
+            clipboard.setText(text)
+
+        menu.addSection('Copy to clipboard')
+        action = menu.addAction('&All')
+        action.triggered.connect(functools.partial(copy_to_clipboard,
+                                                   text=self.toolTip()))
+        menu.addSeparator()
+
+        for line in self.toolTip().splitlines():
+            action = menu.addAction(line)
+            action.triggered.connect(
+                functools.partial(copy_to_clipboard, text=line)
+            )
+
+        menu.exec_(self.mapToGlobal(event.pos()))
+
     def _handle_timeout(self):
         self._animation.stop()
         self.setMovie(None)
-        self.setText("Loading Timeout")
-        self.setToolTip("Could not complete operation after "
-                        f"{self.LOADING_TIMEOUT_MS/1000} seconds.")
+        self.setText(self.timeout_message)
 
     @property
     def iconSize(self):
@@ -423,14 +459,14 @@ def remove_duplicate_items(list_):
 
 def is_standard_template(template):
     """
-    Is the template one provided with typhos?
+    Is the template a core one provided with typhos?
 
     Parameters
     ----------
     template : str or pathlib.Path
     """
-    common_path = pathlib.Path(os.path.commonpath((template, MODULE_PATH)))
-    return common_path == MODULE_PATH
+    common_path = pathlib.Path(os.path.commonpath((template, ui_core_dir)))
+    return common_path == ui_core_dir
 
 
 def find_templates_for_class(cls, view_type, paths, *, extensions=None,
@@ -1048,3 +1084,176 @@ def dump_grid_layout(layout, rows=None, cols=None, *, cell_width=60):
 def nullcontext():
     """Stand-in for py3.7's contextlib.nullcontext"""
     yield
+
+
+def get_component(obj):
+    """
+    Get the component that made the given object.
+
+    Parameters
+    ----------
+    obj : ophyd.OphydItem
+        The ophyd item for which to get the component.
+
+    Returns
+    -------
+    component : ophyd.Component
+        The component, if available.
+    """
+    if obj.parent is None:
+        return None
+
+    return getattr(type(obj.parent), obj.attr_name, None)
+
+
+def get_variety_metadata(cpt):
+    """
+    Get "variety" metadata from a component or signal.
+
+    Parameters
+    ----------
+    cpt : ophyd.Component or ophyd.OphydItem
+        The component / ophyd item to get the metadata for.
+
+    Returns
+    -------
+    metadata : dict
+        The metadata, if set. Otherwise an empty dictionary.  This metadata is
+        guaranteed to be valid according to the known schemas.
+    """
+    if not isinstance(cpt, ophyd.Component):
+        cpt = get_component(cpt)
+
+    return getattr(cpt, '_variety_metadata', {})
+
+
+def widget_to_image(widget, fill_color=QtCore.Qt.transparent):
+    """
+    Paint the given widget in a new QtGui.QImage.
+
+    Returns
+    -------
+    QtGui.QImage
+        The display, as an image.
+    """
+    image = QtGui.QImage(widget.width(), widget.height(),
+                         QtGui.QImage.Format_ARGB32_Premultiplied)
+
+    image.fill(fill_color)
+    pixmap = QtGui.QPixmap(image)
+
+    painter = QtGui.QPainter(pixmap)
+    widget.render(image)
+    painter.end()
+    return image
+
+
+_connect_slots_unpatched = None
+
+
+def patch_connect_slots():
+    """
+    Patches QtCore.QMetaObject.connectSlotsByName to catch SystemErrors.
+    """
+    global _connect_slots_unpatched
+
+    if _connect_slots_unpatched is not None:
+        return
+
+    # TODO there could be a version check here if we can isolate it
+
+    _connect_slots_unpatched = QtCore.QMetaObject.connectSlotsByName
+
+    def connect_slots_patch(top_level_widget):
+        try:
+            return _connect_slots_unpatched(top_level_widget)
+        except SystemError as ex:
+            logger.debug(
+                "Eating system error.  This may possibly be solved by either "
+                "downgrading Python or upgrading pyqt5 to >= 5.13.1. "
+                "For further discussion, see "
+                "https://github.com/pcdshub/typhos/issues/354",
+                exc_info=ex
+            )
+
+    QtCore.QMetaObject.connectSlotsByName = connect_slots_patch
+
+
+def link_signal_to_widget(signal, widget):
+    """
+    Registers the signal with PyDM, and sets the widget channel.
+
+    Parameters
+    ----------
+    signal : ophyd.OphydObj
+        The signal to use.
+
+    widget : QtWidgets.QWidget
+        The widget with which to connect the signal.
+    """
+    if signal is not None:
+        plugins.register_signal(signal)
+        if widget is not None:
+            read = not isinstance(widget, PyDMWritableWidget)
+            widget.channel = channel_from_signal(signal, read=read)
+
+
+def linked_attribute(property_attr, widget_attr, hide_unavailable=False):
+    """
+    Decorator which connects a device signal with a widget.
+
+    Retrieves the signal from the device, registers it with PyDM, and sets the
+    widget channel.
+
+    Parameters
+    ----------
+    property_attr : str
+        This is one level of indirection, allowing for the component attribute
+        to be configurable by way of designable properties.
+        In short, this looks like:
+        ``getattr(self.device, getattr(self, property_attr))``
+        The component attribute name may include multiple levels (e.g.,
+        ``'cpt1.cpt2.low_limit'``).
+
+    widget_attr : str
+        The attribute name of the widget, referenced from ``self``.
+        The component attribute name may include multiple levels (e.g.,
+        ``'ui.low_limit'``).
+
+    hide_unavailable : bool
+        Whether or not to hide widgets for which the device signal is not
+        available
+    """
+    get_widget_attr = operator.attrgetter(widget_attr)
+
+    def wrapper(func):
+        @functools.wraps(func)
+        def wrapped(self):
+            widget = get_widget_attr(self)
+            device_attr = getattr(self, property_attr)
+            get_device_attr = operator.attrgetter(device_attr)
+
+            try:
+                signal = get_device_attr(self.device)
+            except AttributeError:
+                signal = None
+            else:
+                # Fall short of an `isinstance(signal, OphydObj) check here:
+                try:
+                    link_signal_to_widget(signal, widget)
+                except Exception:
+                    logger.exception(
+                        'device.%s => self.%s (signal: %s widget: %s)',
+                        device_attr, widget_attr, signal, widget)
+                    signal = None
+                else:
+                    logger.debug('device.%s => self.%s (signal=%s widget=%s)',
+                                 device_attr, widget_attr, signal, widget)
+
+            if signal is None and hide_unavailable:
+                widget.setVisible(False)
+
+            return func(self, signal, widget)
+
+        return wrapped
+    return wrapper
